/*
 * database.c
 ******************************************************************************
 * Copyright (C) 2002 Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#include <stdio.h>
#include <string.h> // for memset
#include <ctype.h> // for tolower
#include "types.h"
#include "database.h"

#ifdef UCLINUX
#include "process.h"
#define fprintf PR_log_msg
#define stderr  ERR
#endif
/******************************************************************************
 */

const STRING_CLASS DB_obj_type[] =
{ /**** ONLY ADD TO END OF THIS LIST!!! ****/
  {"",				"Unknown object type"},

  {"tripple,trpl,d**,***",	"Tripple star"},
  {"double,binary,dbl,d*,**",	"Double star"},
  {"visual,optical,astronometric","Visual (astronometric) binary"},

  {"globular,gb,glob",		"Globular cluster"},
  {"open,oc",			"Open cluster"},
  {"c+n",			"Cluster associated with nebulosity"},

  {"emission,eneb",		"Emission nebula"},
  {"reflection,rneb",		"Reflection nebula"},
  {"planetary,pl,pneb",		"Planetary nebula"},
  {"diffuse,dneb",		"Diffuse nebula"},
  {"supernova",			"Remnant of a supernova"},
  {"kt",			"Nebulous region (knot) in a galaxy"},

  {"spiral",			"Spiral (S/SA) (SB) galaxy"},
  {"elliptical",		"Elliptical (E) galaxy"},
  {"irregular",			"Irregular (I) galaxy"},
  {"lenticular",		"Lenticular (S0) galaxy"},

  {"@boundary",			"You shouldn't see this"},

  {"multiple,mstar",		"Unclassified multiple star"},
  {"star,sngl,*",		"Star"},
  {"cluster",			"Unclassified cluster"},
  {"nebula,nb,neb",		"Unclassified nebula"},
  {"galaxy,gx,gc,gal,igal",	"Unclassified galaxy"},
  {"asterism,group,ast",	"Group or Asterism"},

  {(char *)0 /* end of table */}
} ;


const STRING_CLASS DB_constellations[] =
{ /**** ONLY ADD TO END OF THIS LIST!!! ****/
  {"",				"Unknown constellation"},

  {"andromeda,and",		"Andromeda"},
  {"antlia,ant",		"Antlia"},
  {"apus,aps",			"Apus"},
  {"aquarius,aqr",		"Aquarius"},
  {"aquila,aql",		"Aquila"},
  {"ara",			"Ara"},
  {"aries,ari",			"Aries"},
  {"auriga,aur",		"Auriga"},
  {"bootes,boo",		"Bootes"},
  {"caelum,cae",		"Caelum"},
  {"camelopardalis,cam",	"Camelopardalis"},
  {"cancer,cnc",		"Cancer"},
  {"canes venatici,cvn",	"Canes Venatici"},
  {"canis major,cma",		"Canis Major"},
  {"canis minor,cmi",		"Canis Minor"},
  {"capricornus,cap",		"Capricornus"},
  {"carina,car",		"Carina"},
  {"cassiopeia,cas",		"Cassiopeia"},
  {"centaurus,cen",		"Centaurus"},
  {"cepheus,cep",		"Cepheus"},
  {"cetus,cet",			"Cetus"},
  {"chamaeleon,cha",		"Chamaeleon"},
  {"circinus,cir",		"Circinus"},
  {"columba,col",		"Columba"},
  {"coma berenices,com",	"Coma Berenices"},
  {"corona austrina,cra",	"Corona Austrina"},
  {"corona borealis,crb",	"Corona Borealis"},
  {"corvus,crv",		"Corvus"},
  {"crater,crt",		"Crater"},
  {"crux,cru",			"Crux"},
  {"cygnus,cyg",		"Cygnus"},
  {"delphinus,del",		"Delphinus"},
  {"dorado,dor",		"Dorado"},
  {"draco,dra",			"Draco"},
  {"equuleus,equ",		"Equuleus"},
  {"eridanus,eri",		"Eridanus"},
  {"fornax,for",		"Fornax"},
  {"gemini,gem",		"Gemini"},
  {"grus,gru",			"Grus"},
  {"hercules,her",		"Hercules"},
  {"horologium,hor",		"Horologium"},
  {"hydra,hya",			"Hydra"},
  {"hydrus,hyi",		"Hydrus"},
  {"indus,ind",			"Indus"},
  {"lacerta,lac",		"Lacerta"},
  {"leo",			"Leo"},
  {"leo minor,lmi",		"Leo Minor"},
  {"lepus,lep",			"Lepus"},
  {"libra,lib",			"Libra"},
  {"lupus,lup",			"Lupus"},
  {"lynx,lyn",			"Lynx"},
  {"lyra,lyr",			"Lyra"},
  {"mensa,men",			"Mensa"},
  {"microscopium,mic",		"Microscopium"},
  {"monoceros,mon",		"Monoceros"},
  {"musca,mus",			"Musca"},
  {"norma,nor",			"Norma"},
  {"octans,oct",		"Octans"},
  {"ophiuchus,oph",		"Ophiuchus"},
  {"orion,ori",			"Orion"},
  {"pavo,pav",			"Pavo"},
  {"pegasus,peg",		"Pegasus"},
  {"perseus,per",		"Perseus"},
  {"phoenix,phe",		"Phoenix"},
  {"pictor,pic",		"Pictor"},
  {"pisces,psc",		"Pisces"},
  {"piscis austrinus,psa",	"Piscis Austrinus"},
  {"puppis,pup",		"Puppis"},
  {"pyxis,pyx",			"Pyxis"},
  {"reticulum,ret",		"Reticulum"},
  {"sagitta,sge",		"Sagitta"},
  {"sagittarius,sgr",		"Sagittarius"},
  {"scorpius,sco",		"Scorpius"},
  {"sculptor,scl",		"Sculptor"},
  {"scutum,sct",		"Scutum"},
  {"serpens,ser",		"Serpens"},
  {"sextans,sex",		"Sextans"},
  {"taurus,tau",		"Taurus"},
  {"telescopium,tel",		"Telescopium"},
  {"triangulum,tri",		"Triangulum"},
  {"triangulum australe,tra",	"Triangulum Australe"},
  {"tucana,tuc",		"Tucana"},
  {"ursa major,uma",		"Ursa Major"},
  {"ursa minor,umi",		"Ursa Minor"},
  {"vela,vel",			"Vela"},
  {"virgo,vir",			"Virgo"},
  {"volans,vol",		"Volans"},
  {"vulpecula,vul",		"Vulpecula"},

  {(char *)0 /* end of table */}
} ;


const STRING_CLASS DB_seasons[] =
{ /**** ONLY ADD TO END OF THIS LIST!!! ****/
  {"",				"Unknown season"},

  {"autumn/early-winter,autumn/early winter,"
   "autumn / early-winter,autumn / early winter,"
   "fall/early-winter,fall/early winter,"
   "fall / early-winter,fall / early winter",
		"Late autumn / early winter"},

  {"early-winter,early winter",	"Early winter"},
  {"mid-winter,mid winter",	"Mid winter"},
  {"late-winter,late winter",	"Late winter"},
  {"winter",			"Winter"},

  {"winter/early-spring,winter/early spring,"
   "winter / early-spring,winter / early spring",
		"Late winter / early spring"},

  {"early-spring,early spring",	"Early spring"},
  {"mid-spring,mid spring",	"Mid spring"},
  {"late-spring,late spring",	"Late spring"},
  {"spring",			"Spring"},

  {"spring/early-summer,spring/early summer,"
   "spring / early-summer,spring / early summer",
		"Late spring / early summer"},

  {"early-summer,early summer",	"Early summer"},
  {"mid-summer,mid summer",	"Mid summer"},
  {"late-summer,late summer",	"Late summer"},
  {"summer",			"Summer"},

  {"summer/early-autumn,summer/early autumn,"
   "summer/early-fall,summer/early fall,"
   "summer / early-autumn,summer / early autumn,"
   "summer / early-fall,summer / early fall",
		"Late summer / early autumn"},

  {"early-autumn,early autumn,early-fall,early fall",	"Early autumn"},
  {"mid-autumn,mid autumn,mid-fall,mid fall",		"Mid autumn"},
  {"late-autumn,late autumn,late-fall,late fall",	"Late autumn"},
  {"autumn,fall",					"Autumn"},

  {(char *)0 /* end of table */}
} ;

const STRING_CLASS DB_rating_type[] =
{ /**** ONLY ADD TO END OF THIS LIST!!! ****/
  {"",		"Unknown rating"},

  {"su,a,!!!",	"Superb (A)"},
  {"ex,b,!!",	"Excellent (B)"},
  {"vg,c,!",	"Very good (C)"},
  {"gd,d",	"Good (D)"},
  {"fr,e",	"Fair (E)"},
  {"pr,f",	"Poor (F)"},
  {"vp,g",	"Very poor (G)"},

  {(char *)0 /* end of table */}
} ;

/******************************************************************************
 * The following routines are used to read, write and display pre-assembled
 * object to a databases
 */

#ifdef DB_WRITE

void
write_number (FILE *f, Ushort num)
{
  fputc (num & 0xff, f) ;
  fputc ((num >> 8) & 0xff, f) ;
}

void
DB_write_header (FILE *f, NUM_ENTRIES *num)
{
  write_number (f, (Ushort)(num->ngc)) ;
  write_number (f, (Ushort)(num->ic)) ;
  write_number (f, (Ushort)(num->ugc)) ;
  write_number (f, (Ushort)(num->hr)) ;
  write_number (f, (Ushort)(num->sao)) ;
  write_number (f, (Ushort)(num->pgc)) ;
  write_number (f, (Ushort)(num->hd)) ;
  write_number (f, (Ushort)(num->ppm)) ;
  write_number (f, (Ushort)(num->messier)) ;
  write_number (f, (Ushort)(num->align)) ;
}

int
DB_write_object_info (const OBJECT_INFO *o, FILE *f)
{
  Uchar x[sizeof(OBJECT_INFO)] ;
  Ushort i ;
  int j ;

  i = 0 ;

  // Firstly, write RA, DEC and bit field information
  // RA and DEC are 20 bit numbers stored in 5 bytes, MSB first
  x[i++] = (o->ra >> 12) & 0xff ;
  x[i++] = (o->ra >> 4) & 0xff ;
  x[i++] = ((o->ra << 4) & 0xf0) | (((Ulong)o->dec >> 16) & 0x0f) ;
  x[i++] = ((Ulong)o->dec >> 8) & 0xff ;
  x[i++] = (Ulong)o->dec & 0xff ;

  x[i++] = (o->bit_fields & 0xff000000) >> 24 ;
  x[i++] = (o->bit_fields & 0x00ff0000) >> 16 ;
  x[i++] = (o->bit_fields & 0x0000ff00) >> 8 ;
  x[i++] =  o->bit_fields & 0x000000ff ;

  if (o->bit_fields & BIT_MESSIER)
    x[i++] = o->messier ;

  if (o->bit_fields & BIT_NGC)
  {
    x[i++] = (o->ngc >> 8) & 0xff ;
    x[i++] = o->ngc & 0xff ;
  }

  if (o->bit_fields & BIT_NGC_COMP)
    x[i++] = tolower(o->ngc_comp) ;

  if (o->bit_fields & BIT_UGC)
  {
    x[i++] = (o->ugc >> 16) & 0xff ;
    x[i++] = (o->ugc >> 8) & 0xff ;
    x[i++] = o->ugc & 0xff ;
  }

  if (o->bit_fields & BIT_UGC_COMP)
    x[i++] = tolower(o->ugc_comp) ;

  if (o->bit_fields & BIT_HR)
  {
    x[i++] = (o->hr >> 8) & 0xff ;
    x[i++] = o->hr & 0xff ;
  }

  if (o->bit_fields & BIT_IC)
  {
    x[i++] = (o->ic >> 8) & 0xff ;
    x[i++] = o->ic & 0xff ;
  }

  if (o->bit_fields & BIT_IC_COMP)
    x[i++] = tolower(o->ic_comp) ;

  if (o->bit_fields & BIT_SAO)
  {
    x[i++] = (o->sao >> 16) & 0xff ;
    x[i++] = (o->sao >> 8) & 0xff ;
    x[i++] = o->sao & 0xff ;
  }

  if (o->bit_fields & BIT_PGC)
  {
    x[i++] = (o->pgc >> 16) & 0xff ;
    x[i++] = (o->pgc >> 8) & 0xff ;
    x[i++] = o->pgc & 0xff ;
  }

  if (o->bit_fields & BIT_HD)
  {
    x[i++] = (o->hd >> 16) & 0xff ;
    x[i++] = (o->hd >> 8) & 0xff ;
    x[i++] = o->hd & 0xff ;
  }

  if (o->bit_fields & BIT_PPM)
  {
    x[i++] = (o->ppm >> 16) & 0xff ;
    x[i++] = (o->ppm >> 8) & 0xff ;
    x[i++] = o->ppm & 0xff ;
  }

  if (o->bit_fields & BIT_CONST)
    x[i++] = o->constellation ;

  if (o->bit_fields & BIT_LY)
  {
    x[i++] = (o->ly >> 24) & 0xff ;
    x[i++] = (o->ly >> 16) & 0xff ;
    x[i++] = (o->ly >> 8) & 0xff ;
    x[i++] = o->ly & 0xff ;
  }

  if (o->bit_fields & BIT_RATING)
    x[i++] = o->rating ;

  if (o->bit_fields & BIT_SEASON)
    x[i++] = o->season ;

  if (o->bit_fields & BIT_TYPE)
    x[i++] = o->type ;

  if (o->bit_fields & BIT_ANGLE)
    x[i++] = o->angle ;

  if (o->bit_fields & BIT_MAG)
    x[i++] = o->mag ;

  if (o->bit_fields & BIT_MAG_DIFF)
    x[i++] = o->mag_diff ;

  if (o->bit_fields & BIT_RADIUS)
  {
    x[i++] = (o->radius >> 8) & 0xff ;
    x[i++] = o->radius & 0xff ;
  }

  if (o->bit_fields & BIT_SEP)
  {
    x[i++] = (o->sep >> 8) & 0xff ;
    x[i++] = o->sep & 0xff ;
  }

  if (o->bit_fields & BIT_MINOR)
    x[i++] = o->minor ;

  if (o->bit_fields & BIT_MAJOR)
    x[i++] = o->major ;

  if (o->bit_fields & BIT_NUM_COMP)
    x[i++] = o->num_comp ;

  if (o->bit_fields & BIT_COMMON_NAME)
  {
    j = sprintf (&x[i], "%s", o->common_name) ;

    if (j > (BUFFER_MAX_SIZE-100))
      fprintf (stderr, "ERROR: Writing oversized common name. %d bytes\n", j) ;

    if (j)
      i += j+1 ;
    else
      x[i++] = '\0' ;
  }

  if (o->bit_fields & BIT_FORMAL_NAME)
  {
    j = sprintf (&x[i], "%s", o->formal_name) ;

    if (j > (BUFFER_MAX_SIZE-100))
      fprintf (stderr, "ERROR: Writing oversized formal name. %d bytes\n", j) ;

    if (j)
      i += j+1 ;
    else
      x[i++] = '\0' ;
  }

  if (o->bit_fields & BIT_DESCRIPTION)
  {
    j = sprintf (&x[i], "%s", o->description) ;

    if (j > (BUFFER_MAX_SIZE-100))
      fprintf (stderr, "ERROR: Writing oversized description. %d bytes\n", j) ;

    if (j)
      i += j+1 ;
    else
      x[i++] = '\0' ;
  }

  if (o->bit_fields & BIT_SPECTRUM)
  {
    j = sprintf (&x[i], "%s", o->spectrum) ;

    if (j > (BUFFER_MAX_SIZE-100))
      fprintf (stderr, "ERROR: Writing oversized spectrum. %d bytes\n", j) ;

    if (j)
      i += j+1 ;
    else
      x[i++] = '\0' ;
  }

  if ((o->bit_fields & BIT_RA_APM) && o->ra_apm)
    x[i++] = o->ra_apm ;

  if ((o->bit_fields & BIT_DEC_APM) && (o->dec_apm))
    x[i++] = o->dec_apm ;

  return fwrite (x, 1, i, f) ; // write i*single byte elements of array "x"
}

#endif


/******************************************************************************
 */

Ushort
DB_read_number (FILE *f)
{
  Ushort x, y ;

  y = (Ushort)fgetc (f) ;
  x = ((Ushort)fgetc(f)) << 8 ;
  return x | y ;
}

void
DB_read_header (FILE *f, NUM_ENTRIES *num)
{
  num->ngc = (Ulong)DB_read_number (f) ;
  num->ic = (Ulong)DB_read_number (f) ;
  num->ugc = (Ulong)DB_read_number (f) ;
  num->hr = (Ulong)DB_read_number (f) ;
  num->sao = (Ulong)DB_read_number (f) ;
  num->pgc = (Ulong)DB_read_number (f) ;
  num->hd = (Ulong)DB_read_number (f) ;
  num->ppm = (Ulong)DB_read_number (f) ;
  num->messier = (Ulong)DB_read_number (f) ;
  num->align = (Ulong)DB_read_number (f) ;
}

int
DB_read_object_info (OBJECT_INFO *o, FILE *f, const int brief)
{
  Uchar x[5] ;
  Ushort i ;
  int j ;

  memset (o, '\0', sizeof(OBJECT_INFO)) ;
  i = 0 ;
  o->common_name = o->formal_name = o->description = o->spectrum = o->buffer ;

  // Firstly, read RA, DEC and bit b information
  // RA and DEC are 20 bit numbers stored in 5 bytes, MSB first
  if (fread (x, 1, 5, f) != 5)
    return 0 ;

  o->ra = (x[0] << 12) | (x[1] << 4) | ((x[2] & 0xf0) >> 4) ;
  o->dec = (long)(((x[2] & 0x0f) << 16) | (x[3] << 8) | x[4]) ;

  if (o->dec & 0x00080000)
    o->dec |= 0xfff00000 ;

  // Now read the bit b telling us what information follows
  if (fread (x, 1, 4, f) != 4)
    return 0 ;

  o->bit_fields = (x[0] << 24) | (x[1] << 16) | (x[2] << 8) | x[3] ;

  if (o->bit_fields & BIT_MESSIER)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->messier = x[0] ;
  }

  if (o->bit_fields & BIT_NGC)
  {
    if (fread (x, 1, 2, f) != 2)
      return 0 ;

    o->ngc = (x[0] << 8) | x[1] ;
  }

  if (o->bit_fields & BIT_NGC_COMP)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->ngc_comp = x[0] ;
  }

  if (o->bit_fields & BIT_UGC)
  {
    if (fread (x, 1, 3, f) != 3)
      return 0 ;

    o->ugc = (x[0] << 16) | (x[1] << 8) | x[2] ;
  }

  if (o->bit_fields & BIT_UGC_COMP)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->ugc_comp = x[0] ;
  }

  if (o->bit_fields & BIT_HR)
  {
    if (fread (x, 1, 2, f) != 2)
      return 0 ;

    o->hr = (x[0] << 8) | x[1] ;
  }

  if (o->bit_fields & BIT_IC)
  {
    if (fread (x, 1, 2, f) != 2)
      return 0 ;

    o->ic = (x[0] << 8) | x[1] ;
  }

  if (o->bit_fields & BIT_IC_COMP)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->ic_comp = x[0] ;
  }

  if (o->bit_fields & BIT_SAO)
  {
    if (fread (x, 1, 3, f) != 3)
      return 0 ;

    o->sao = (x[0] << 16) | (x[1] << 8) | x[2] ;
  }

  if (o->bit_fields & BIT_PGC)
  {
    if (fread (x, 1, 3, f) != 3)
      return 0 ;

    o->pgc = (x[0] << 16) | (x[1] << 8) | x[2] ;
  }

  if (o->bit_fields & BIT_HD)
  {
    if (fread (x, 1, 3, f) != 3)
      return 0 ;

    o->hd = (x[0] << 16) | (x[1] << 8) | x[2] ;
  }

  if (o->bit_fields & BIT_PPM)
  {
    if (fread (x, 1, 3, f) != 3)
      return 0 ;

    o->ppm = (x[0] << 16) | (x[1] << 8) | x[2] ;
  }

  if (brief)
    return 1 ;

  if (o->bit_fields & BIT_CONST)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->constellation = x[0] ;
  }

  if (o->bit_fields & BIT_LY)
  {
    if (fread (x, 1, 4, f) != 4)
      return 0 ;

    o->ly = (x[0] << 24) | (x[1] << 16) | (x[2] << 8) | x[3] ;
  }

  if (o->bit_fields & BIT_RATING)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->rating = x[0] ;
  }

  if (o->bit_fields & BIT_SEASON)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->season = x[0] ;
  }

  if (o->bit_fields & BIT_TYPE)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->type = x[0] ;
  }

  if (o->bit_fields & BIT_ANGLE)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->angle = x[0] ;
  }

  if (o->bit_fields & BIT_MAG)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->mag = x[0] ;
  }

  if (o->bit_fields & BIT_MAG_DIFF)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->mag_diff = x[0] ;
  }

  if (o->bit_fields & BIT_RADIUS)
  {
    if (fread (x, 1, 2, f) != 2)
      return 0 ;

    o->radius = (x[0] << 8) | x[1] ;
  }

  if (o->bit_fields & BIT_SEP)
  {
    if (fread (x, 1, 2, f) != 2)
      return 0 ;

    o->sep = (x[0] << 8) | x[1] ;
  }

  if (o->bit_fields & BIT_MINOR)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->minor = x[0] ;
  }

  if (o->bit_fields & BIT_MAJOR)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->major = x[0] ;
  }

  if (o->bit_fields & BIT_NUM_COMP)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->num_comp = x[0] ;
  }

  if (o->bit_fields & BIT_COMMON_NAME)
  {
    o->common_name = &o->buffer[i] ;

    for (j=1 ; j != 0 ;)
    {
      if ((j = fgetc(f)) == EOF)
	return 0 ;

      o->buffer[i++] = (char)j ;
    }

    if (strlen(o->common_name) == 0)
      o->bit_fields &= ~BIT_COMMON_NAME ;
    else
      o->formal_name = o->description = o->spectrum = &o->buffer[i-1] ;
  }

  if (o->bit_fields & BIT_FORMAL_NAME)
  {
    o->formal_name = &o->buffer[i] ;

    for (j=1 ; j != 0 ;)
    {
      if ((j = fgetc(f)) == EOF)
	return 0 ;

      o->buffer[i++] = (char)j ;
    }

    if (strlen(o->formal_name) == 0)
      o->bit_fields &= ~BIT_FORMAL_NAME ;
    else
      o->description = o->spectrum = &o->buffer[i-1] ;
  }

  if (o->bit_fields & BIT_DESCRIPTION)
  {
    o->description = &o->buffer[i] ;

    for (j=1 ; j != 0 ;)
    {
      if ((j = fgetc(f)) == EOF)
	return 0 ;

      o->buffer[i++] = (char)j ;
    }

    if (strlen(o->description) == 0)
      o->bit_fields &= ~BIT_DESCRIPTION ;
    else
      o->spectrum = &o->buffer[i-1] ;
  }

  if (o->bit_fields & BIT_SPECTRUM)
  {
    o->spectrum = &o->buffer[i] ;

    for (j=1 ; j != 0 ;)
    {
      if ((j = fgetc(f)) == EOF)
	return 0 ;

      o->buffer[i++] = (char)j ;
    }

    if (strlen(o->spectrum) == 0)
      o->bit_fields &= ~BIT_SPECTRUM ;
  }

  if (o->bit_fields & BIT_RA_APM)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->ra_apm = x[0] ;
  }

  if (o->bit_fields & BIT_DEC_APM)
  {
    if (fread (x, 1, 1, f) != 1)
      return 0 ;

    o->dec_apm = x[0] ;
  }

  if (i > BUFFER_MAX_SIZE)
  {
    fprintf (stderr, "READ_DATABASE: read too many chars and overflowed buffer (%d)\n", i) ;
    fprintf (stderr, "common=%d, formal=%d, descript=%d, spectrum=%d\n",
	     strlen(o->common_name), strlen(o->formal_name),
	     strlen(o->description), strlen(o->spectrum)) ;
    o->description[100]='\0' ;
    fprintf (stderr, "%s", o->description) ;
    return 0 ;
  }

  return 1 ;
}

/******************************************************************************
 */

#ifdef DB_DISPLAY
#include "astro.h"

void
DB_display_object_info (const OBJECT_INFO *o, FILE *f)
{
  char buf[100] ;

  fprintf (f, "RA: %s, ", AS_display_intern_RA (buf, o->ra, 1)) ;
  fprintf (f, "DEC: %s\n", AS_display_intern_DEC (buf, o->dec, 1)) ;

  if (o->bit_fields & BIT_RA_APM)
    fprintf (f, "RA Annual proper motion: %.3f sec\n", o->ra_apm / 300.0) ;

  if (o->bit_fields & BIT_DEC_APM)
    fprintf (f, "DEC Annual proper motion: %.2f arc sec\n", o->dec_apm / 20.0) ;

  if (o->bit_fields & BIT_COMMON_NAME)
    fprintf (f, "Common name: %s\n", o->common_name) ;

  if (o->bit_fields & BIT_FORMAL_NAME)
    fprintf (f, "Formal name: %s\n", o->formal_name) ;

  if (o->bit_fields &
      (BIT_MESSIER | BIT_NGC | BIT_UGC | BIT_HR | BIT_IC | BIT_SAO | BIT_PGC | BIT_HD | BIT_PPM))
  {
    char *x ;

    x = "" ;
    fprintf (f, "Catalogued as: ") ;

    if (o->bit_fields & BIT_MESSIER)
    {
      fprintf (f, "M %lu", (Ulong)o->messier) ;
      x = ", " ;
    }

    if (o->bit_fields & BIT_NGC)
    {
      fprintf (f, "%sNGC %lu", x, (Ulong)o->ngc) ;

      if (o->bit_fields & BIT_NGC_COMP)
	fprintf (f, "%c", o->ngc_comp) ;

      x = ", " ;
    }
    else if (o->bit_fields & BIT_NGC_COMP)
      fprintf (f, "NGC_COMP is set to %c but NGC is empty\n", o->ngc_comp) ;

    if (o->bit_fields & BIT_UGC)
    {
      fprintf (f, "%sUGC %lu", x, (Ulong)o->ugc) ;

      if (o->bit_fields & BIT_UGC_COMP)
	fprintf (f, "%c", o->ugc_comp) ;

      x = ", " ;
    }
    else if (o->bit_fields & BIT_UGC_COMP)
      fprintf (f, "UGC_COMP is set to %c but UGC is empty\n", o->ugc_comp) ;

    if (o->bit_fields & BIT_HR)
    {
      fprintf (f, "%sHR %lu", x, (Ulong)o->hr) ;
      x = ", " ;
    }

    if (o->bit_fields & BIT_IC)
    {
      fprintf (f, "%sIC %lu", x, (Ulong)o->ic) ;

      if (o->bit_fields & BIT_IC_COMP)
	fprintf (f, "%c", o->ic_comp) ;

      x = ", " ;
    }
    else if (o->bit_fields & BIT_IC_COMP)
      fprintf (f, "IC_COMP is set to %c but IC is empty\n", o->ic_comp) ;

    if (o->bit_fields & BIT_SAO)
    {
      fprintf (f, "%sSAO %lu", x, (Ulong)o->sao) ;
      x = ", " ;
    }

    if (o->bit_fields & BIT_PGC)
    {
      fprintf (f, "%sPGC %lu", x, (Ulong)o->pgc) ;
      x = ", " ;
    }

    if (o->bit_fields & BIT_HD)
    {
      fprintf (f, "%sHD %lu", x, (Ulong)o->hd) ;
      x = ", " ;
    }

    if (o->bit_fields & BIT_PPM)
      fprintf (f, "%sPPM %lu", x, (Ulong)o->ppm) ;

    fprintf (f, "\n") ;
  }

  if (o->bit_fields & BIT_ALIGNMENT)
    fprintf (f, "This is an alignment star\n") ;

  if (o->bit_fields & BIT_MAG)
    fprintf (f, "Magnitude: %01.1f\n", (o->mag - 30) / 10.0) ;

  if (o->bit_fields & BIT_LY)
    fprintf (f, "Distance: %1.1f LY\n", o->ly /10.0) ;

  if (o->bit_fields & BIT_TYPE)
    fprintf (f, "Type: %s\n", DB_obj_type[o->type].description) ;

  if (o->bit_fields & BIT_NUM_COMP)
    fprintf (f, "Number of components: %lu\n", (Ulong)o->num_comp) ;

  if (o->bit_fields & BIT_MAG_DIFF)
    fprintf (f, "Magnitude difference: %1.1f\n", o->mag_diff / 10.0) ;

  if (o->bit_fields & BIT_SEP)
    fprintf (f, "Separation: %lu arc sec\n", (Ulong)o->sep) ;

  if (o->bit_fields & BIT_RADIUS)
    fprintf (f, "Radius: %lu solar units\n", (Ulong)o->radius) ;

  if (o->bit_fields & BIT_MAJOR)
  {
    fprintf (f, "Apparent size: %lu", (Ulong)o->major) ;

    if (o->bit_fields & BIT_MINOR)
      fprintf (f, " x %lu", (Ulong)o->minor) ;

    fprintf (f, " arc seconds\n") ;
  }
  else if (o->bit_fields & BIT_MINOR)
    fprintf (f, "MINOR set but MAJOR not set: value=%lu\n", (Ulong)o->minor) ;

  if (o->bit_fields & BIT_CONST)
    fprintf (f, "Constellation: %s\n", DB_constellations[o->constellation].description) ;

  if (o->bit_fields & BIT_RATING)
    fprintf (f, "Subjective rating: %s\n", DB_rating_type[o->rating].description) ;

  if (o->bit_fields & BIT_SEASON)
    fprintf (f, "Season: %s\n", DB_seasons[o->season].description) ;

  if (o->bit_fields & BIT_ANGLE)
    fprintf (f, "Angle class: %lu\n", (Ulong)o->angle) ;

  if (o->bit_fields & BIT_SPECTRUM)
    fprintf (f, "Spectrum: %s\n", o->spectrum) ;

  if (o->bit_fields & BIT_DESCRIPTION)
    fprintf (f, "Description: %s\n", o->description) ;
}

#endif
