/*
 * test_astro.c
 ******************************************************************************
 * Copyright (C) 2002 Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#include <stdio.h>
#include <math.h>
#include <float.h>

#include "types.h"
#include "astro.h"

struct test_jday
{
  AS_TIME_STRUCT t ;
  double	jd ;
  double	sidereal ;
} ;

#define CONV(YEAR, MON, DATE, HOUR, MIN, SEC)	{YEAR, MON-1, DATE, HOUR, MIN, SEC}

char buf[100], buf1[100] ;

struct test_jday T[] =
{
  {CONV(1957,10,4,19,26,24), 2436116.31},
  {CONV(1987,1,27,0,0,0), 2446822.5},
  {CONV(1998,2,26,0,0,0), 0},
  {CONV(1998,2,27,0,0,0), 0},
  {CONV(1998,2,28,0,0,0), 0},
  {CONV(1998,3,1,0,0,0), 0},
  {CONV(1998,3,2,0,0,0), 0},
  {CONV(1999,1,1,0,0,0), 0},
  {CONV(1999,2,27,0,0,0), 0},
  {CONV(1999,2,28,0,0,0), 0},
  {CONV(1999,3,1,0,0,0), 0},
  {CONV(1999,3,2,0,0,0), 0},
  {CONV(2000,1,1,12,0,0), 2451545.0},
  {CONV(2000,2,27,12,0,0), 0},
  {CONV(2000,2,28,12,0,0), 0},
  {CONV(2000,2,29,12,0,0), 0},
  {CONV(2000,3,1,12,0,0), 0},
  {CONV(2000,3,2,12,0,0), 0},
  {CONV(2001,2,27,12,0,0), 0},
  {CONV(2001,2,28,12,0,0), 0},
  {CONV(2001,3,1,12,0,0), 0},

  {CONV(1987,1,27,0,0,0), 2446822.5},
  {CONV(1987,6,15,12,0,0), 2446962.0},
  {CONV(1988,1,27,0,0,0), 2447187.5},
  {CONV(1988,6,19,12,0,0), 2447332.0},
  {CONV(1998,12,31,12,0,0), 2451179.0},
  {CONV(1999,1,1,0,0,0), 2451179.5},
  {CONV(1999,1,1,12,0,0), 2451180.0},
  {CONV(1999,2,27,12,0,0), 2451237.0},
  {CONV(1999,2,28,0,0,0), 2451237.5},
  {CONV(1999,2,28,12,0,0), 2451238.0},
  {CONV(1999,3,1,0,0,0), 2451238.5},
  {CONV(2000,1,1,12,0,0), 2451545.0},

  {CONV(1987,4,10,0,0,0), 0},
  {CONV(1987,4,10,19,21,0), 0},

  {CONV(0,0,0,0,0,0),-1}
} ;
///////////////////////////////////////////////////////////////////////////////
void
display (const char *name, const ASH_POSITION_STRUCT *pos)
{
  printf ("%7s: Raw position: %s %s\n", name, AS_rad_to_HMS (buf, pos->obj_ra), AS_rad_to_DMS (buf1, pos->obj_dec)) ;
  printf ("%7s: Apa position: %s %s\n", name, AS_rad_to_HMS (buf, pos->apa_ra), AS_rad_to_DMS (buf1, pos->apa_dec)) ;
  printf ("%7s: Distance:     %.7f light day\n", name, pos->apa_distance * 0.0057755183) ;
  printf ("%7s: Alt=%s Az=%s\n", name, AS_rad_to_DMS (buf, pos->altitude), AS_rad_to_DMS (buf1, pos->azimuth)) ;
  printf ("%7s: Ref position: %s %s\n", name, AS_rad_to_HMS (buf, pos->apa_ra+pos->delta_ra), AS_rad_to_DMS (buf1, pos->apa_dec+pos->delta_dec)) ;
  printf ("%7s: Refraction  : %s %s\n\n", name, AS_rad_to_HMS (buf, pos->delta_ra), AS_rad_to_DMS (buf1, pos->delta_dec)) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_float (char *name, double a, double b, double mult)
{
  int x, y ;

  x = a*mult + 0.5 ;
  y = b*mult + 0.5 ;

  if (x != y)
    printf ("Error! %s calculated as %f. Should have been %f\n", name, a, b) ;
}
///////////////////////////////////////////////////////////////////////////////
void
check_pi()
{
  double x ;

  printf ("Number of digits of precision: single=%d, double=%d, long double=%d.\n", FLT_DIG, DBL_DIG, LDBL_DIG) ;
  printf ("constant pi  =%.*f\n", DBL_DIG, M_PI) ;

  x = 4.0 * atan(1.0) ;
  printf ("calculated pi=%.*f\n\n", DBL_DIG, x) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_jd()
{
  Ushort i ;

  printf ("\n\n\nTESTING conversion to Julian Date\n\n") ;

  for (i=0 ; T[i].jd >= 0 ; i++)
  {
    AS_TIME_STRUCT ts ;
    double y ;

    y = AS_julian_day (&T[i].t) ;
    printf ("%s = JD %.2f\n", AS_display_date(buf, &T[i].t), y) ;

    if (T[i].jd == 0)
      T[i].jd = y ;
    else
      test_float ("T[]", T[i].jd, y, 100000.0) ;

    AS_date_from_julian (T[i].jd, &ts) ;

    if ((T[i].t.date != ts.date) || (T[i].t.month != ts.month) || (T[i].t.year != ts.year)
	|| (T[i].t.hour != ts.hour) || (T[i].t.minute != ts.minute) || (T[i].t.seconds != ts.seconds))
      printf ("\tConversion from JD to date failed (%s <> %s)\n", AS_display_date(buf, &T[i].t), AS_display_date(buf1, &ts)) ;
  }
}
///////////////////////////////////////////////////////////////////////////////
void
check_sidtime()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Mean Sidereal Time (p88)\n\n") ;

  // p88
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1987, 3, 10, 0, 0, 0) ;
  AS_greenwich_sidereal_time (&pos) ;
  printf ("1987 April 10 00:00:00 UT ==> %s\n", AS_display_RA (buf, pos.Grnwch_SidTime)) ;
  test_float ("pos.Grnwch_SidTime", pos.Grnwch_SidTime, 3.450397, 100000.0) ;

  // p89
  printf ("\n(p89)\n") ;
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1987, 3, 10, 19, 21, 0) ;
  AS_greenwich_sidereal_time (&pos) ;
  printf ("1987 April 10 19:21:00 UT ==> %s\n", AS_display_RA (buf, pos.Grnwch_SidTime)) ;
  test_float ("pos.Grnwch_SidTime", pos.Grnwch_SidTime, 2.246900, 100000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_nutation()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Nutation calculation (p148)\n\n") ;

  // p148
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1987, 3, 10, 0, 0, 0) ;
  AS_time_functions (&pos) ;
  pos.obj_ra = ASH_HMS_to_rad(2,46,11.331) ;
  pos.obj_dec = ASH_DMS_to_rad(49,20,54.54) ;
  AS_nutation (&pos) ;
  printf ("pos.delta_psi     ==> %s\n", AS_rad_to_DMS (buf, pos.delta_psi)) ;
  printf ("pos.delta_epsilon ==> %s\n", AS_rad_to_DMS (buf, pos.delta_epsilon)) ;
  printf ("pos.epsilon0      ==> %s\n", AS_rad_to_DMS (buf, pos.epsilon0)) ;
  printf ("pos.e             ==> %s\n", AS_rad_to_DMS (buf, pos.e)) ;
  test_float ("pos.delta_psi", pos.delta_psi, -3.788/3600.0/360.0*2.0*M_PI, 100000.0) ;
  test_float ("pos.delta_epsilon", pos.delta_epsilon, AS_deg_to_rad(9.443 / 3600.0), 100000.0) ;
  test_float ("pos.epsilon0", pos.epsilon0, AS_deg_to_rad(23 + 26/60.0 + 27.407/3600.0), 100000.0) ;
  test_float ("pos.e", pos.e, AS_deg_to_rad(23 + 26/60.0 + 36.850/3600.0), 100000.0) ;
  printf ("\n") ;

  // p152
  printf ("(p152)\n") ;
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 2028, 10, 13, 4, 33, 36) ;
  AS_time_functions (&pos) ;
  pos.obj_ra = ASH_HMS_to_rad(2,46,11.331) ;
  pos.obj_dec = ASH_DMS_to_rad(49,20,54.54) ;
  AS_nutation (&pos) ;
  printf ("pos.delta_psi     ==> %s\n", AS_rad_to_DMS (buf, pos.delta_psi)) ;
  printf ("pos.delta_epsilon ==> %s\n", AS_rad_to_DMS (buf, pos.delta_epsilon)) ;
  printf ("pos.epsilon0      ==> %s\n", AS_rad_to_DMS (buf, pos.epsilon0)) ;
  printf ("pos.e             ==> %s\n", AS_rad_to_DMS (buf, pos.e)) ;
  test_float ("pos.delta_psi", pos.delta_psi, 14.861/3600.0/360.0*2.0*M_PI, 100000.0) ;
  test_float ("pos.delta_epsilon", pos.delta_epsilon, AS_deg_to_rad(2.705/3600.0), 100000.0) ;
  test_float ("pos.epsilon0", pos.epsilon0, AS_deg_to_rad(23.4355), 100000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_sun_lon()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Longitude of the Sun calculation (p165)\n\n") ;

  // p165
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1992, 9, 12, 23, 59, 2) ;
  AS_time_functions (&pos) ;
  AS_lon_sun (&pos) ;
  printf ("pos.lon_sun       ==> %s\n", AS_rad_to_DMS (buf, pos.lon_sun)) ;
  test_float ("pos.lon_sun", pos.lon_sun, AS_deg_to_rad(199.90988), 100000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_ref()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Refraction calculation (p107)\n\n") ;

  // p107
  AS_initialise_struct (&pos) ;
  pos.obj_dec = AS_deg_to_rad (1.246/60.0) ;
  pos.delta_dec = 0.0 ;
  AS_allow_refraction (&pos) ;
  printf ("pos.apa_dec       ==> %s\n", AS_rad_to_DMS (buf, pos.apa_dec+pos.delta_dec)) ;
  test_float ("pos.apa_dec", pos.apa_dec+pos.delta_dec, AS_deg_to_rad(30.0/60.0 + 2.8/3600.0), 100000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_aberation()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Aberration calculation (p152)\n\n") ;

  // p152
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 2028, 10, 13, 4, 33, 36) ;
  AS_time_functions (&pos) ;
  pos.obj_ra = ASH_HMS_to_rad(2,46,11.331) ;
  pos.obj_dec = ASH_DMS_to_rad(49,20,54.54) ;
  AS_nutation (&pos) ;
  AS_aberration (&pos) ;
  printf ("pos.delta_ra      ==> %s\n", AS_rad_to_HMS (buf, pos.delta_ra)) ;
  printf ("pos.delta_dec     ==> %s\n", AS_rad_to_DMS (buf, pos.delta_dec)) ;
  test_float ("pos.delta_ra", pos.delta_ra, AS_deg_to_rad(45.888/3600.0), 100000.0) ;
  test_float ("pos.delta_dec", pos.delta_dec, AS_deg_to_rad(12.91/3600.0), 100000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_moon()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Moon position calculation (p342)\n\n") ;

  //p342
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1992, 3, 11, 23, 59, 2) ;
  AS_find_moon_apparent_location (&pos) ;
  display ("Moon", &pos) ;
  test_float ("pos.obj_ra", pos.obj_ra, AS_deg_to_rad(134.688470), 10000.0) ;
  test_float ("pos.obj_dec", pos.obj_dec, AS_signed_deg_to_rad(13.768368), 10000.0) ;
  test_float ("pos.apa_distance", pos.apa_distance, 368409.7 / 149597870.0, 100000.0) ;

  printf ("\n(p370)\n") ;
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1988, 11, 22, 20, 01, 10) ;
  AS_find_moon_apparent_location (&pos) ;
  display ("Moon", &pos) ;
  test_float ("pos.obj_dec", pos.obj_dec, AS_signed_deg_to_rad(28.1562), 10000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_planet()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Venus position 20th Dec '92, 0h TD (p225-6)\n\n") ;

  //pp225-6
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1992, 11, 19, 23, 59, 2) ;
  AS_find_planet_apparent_location (&pos, VENUS) ;
  display ("Venus", &pos) ;
  test_float ("pos.obj_ra", pos.obj_ra, AS_deg_to_rad(316.17291), 1000.0) ;
  test_float ("pos.obj_dec", pos.obj_dec, AS_signed_deg_to_rad(-18.88801), 10000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_planet_pos()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Planet position at US Naval observatory (p95)\n\n") ;

  //p95
  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 1987, 3, 10, 19, 21, 0) ;
  AS_longitude = AS_deg_to_rad(77.065555556) ;
  AS_lattitude = AS_deg_to_rad(38.921388889) ;
  AS_find_planet_apparent_location (&pos, VENUS) ;
  display ("Venus", &pos) ;
  test_float ("pos.apa_ra", pos.apa_ra, AS_deg_to_rad((23.0+9.0/60.0+16.6/3600.0)*15.0), 100000.0) ;
  test_float ("pos.apa_dec", pos.apa_dec, AS_signed_deg_to_rad(-6.0-43.0/60.0-14.0/3600.0), 100000.0) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_moon_and_planet()
{
  ASH_POSITION_STRUCT pos ;
  Ushort i ;

  printf ("\n\n\nTESTING Moon and planet positions. May 29, 2002. 0000 UT\n\n") ;

  for (i=0 ; i < MAX_PLANETS ; i++)
  {
    AS_initialise_struct (&pos) ;
    AS_fill_time_struct (&pos.GMT, 2002, 4, 29, 0, 0, 0) ;
    AS_find_planet_apparent_location (&pos, i) ;
    display (Planets[i].name, &pos) ;
  }

  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 2002, 4, 29, 0, 0, 0) ;
  AS_find_moon_apparent_location (&pos) ;
  display ("Moon", &pos) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_moon_day()
{
  ASH_POSITION_STRUCT pos ;
  Ushort i ;
  char b[100] ;

  printf ("\n\n\nTESTING MOON during a day (27th May, 2002)\n") ;

  for (i=0 ; i < 24 ; i++)
  {
    AS_initialise_struct (&pos) ;
    if (i < 20)
      AS_fill_time_struct (&pos.GMT, 2002, 4, 27, i+4, 0, 0) ;
    else
      AS_fill_time_struct (&pos.GMT, 2002, 4, 28, i-20, 0, 0) ;

    AS_find_moon_apparent_location (&pos) ;
    printf ("%02hu:00 (Sidereal: %s) ==> %s, %s\n", i, AS_rad_to_HMS (b, pos.Grnwch_SidTime-AS_longitude), AS_rad_to_HMS (buf, pos.obj_ra), AS_rad_to_DMS (buf1, pos.obj_dec)) ;
  }
}
///////////////////////////////////////////////////////////////////////////////
void
test_star_loc()
{
  ASH_POSITION_STRUCT pos ;

  printf ("\n\n\nTESTING Position of polaris in Raleigh, NC\n\n") ;

  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 2002, 4, 25, 0, 0, 0) ; // 8pm Raleigh
  pos.obj_ra = (2.0*3600.0 + 31.0*60.0 + 48.7) / 86400.0 * 2.0 * M_PI ;
  pos.obj_dec = (89.0 + 15.0/60.0 + 51.0/3600.0) / 360.0 * 2.0 * M_PI ;
  AS_find_apparent_location (&pos) ;
  display ("Polaris", &pos) ;
}
///////////////////////////////////////////////////////////////////////////////
void
test_general()
{
  ASH_POSITION_STRUCT pos ;
  char b[100] ;

  printf ("\n\n\nTESTING MOON during a day (27th May, 2002)\n") ;
  printf ("Location = Raleigh, NC. -0400\n\n") ;

  AS_initialise_struct (&pos) ;
  AS_fill_time_struct (&pos.GMT, 2002, 4, 28, 4, 0, 0) ;
  AS_find_moon_apparent_location (&pos) ;
  display ("Moon", &pos) ;
  printf ("%02hu:00 (Sidereal: %s) ==> %s, %s\n", 4, AS_rad_to_HMS (b, pos.Grnwch_SidTime-AS_longitude), AS_rad_to_HMS (buf, pos.obj_ra), AS_rad_to_DMS (buf1, pos.obj_dec)) ;
}
///////////////////////////////////////////////////////////////////////////////
int
main()
{
  AS_longitude = 78.78 / 360.0 * 2.0 * M_PI ;
  AS_lattitude = 35.87 / 360.0 * 2.0 * M_PI ;

  printf ("Longitude ==> %s\n", AS_rad_to_HMS (buf, AS_longitude)) ;
  printf ("Lattitude ==> %s\n", AS_rad_to_DMS (buf, AS_lattitude)) ;
  //  check_pi() ;
  //  test_jd() ;
  // check_sidtime() ;
  //  test_nutation() ;
  //  test_sun_lon() ;
  //  test_ref() ;
  //  test_aberation() ;
  // test_moon() ;
  // test_planet() ;
  // test_planet_pos() ;
  test_moon_and_planet() ;
  // test_moon_day() ;
  // test_star_loc() ;

  return 0 ;
}
