/*
 * asm.s
 ******************************************************************************
 * Copyright (C) 2002  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. http://www.gnu.org
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
	.text
	.even

	.globl supervisor_mode
	.globl user_mode

/******************************************************************************
 * supervisor_mode
 *
 * Entry:	none
 * Exit:	d0 = return parameter (former SSP which contains stack frame
 *			that includes former CC)
 *			Processor is in supervisor mode.
 */
	
supervisor_mode:
	move.l	#0xbc,%a0	/* get address of trap #15 vector */
	lea	int15_loc(%pc),%a1 /* get address of interrupt routine */

	move.l	%a1,(%a0)	/* write interrupt routine to vector */
	trap	#15		/* call interrupt */

int15_loc:
	move.l	%a7,%d0		/* return the old SSP value */
	move.l	%usp,%a7	/* dump old ssp for the usp */
	rts			/* return to caller, now in supervisor mode */

/******************************************************************************
 * user_mode
 *
 * Entry:	d0 = call parameter (former SSP which contains stack frame
 *			that includes former CC)
 * Exit:	Processor is in user mode. The former SSP is restored.
 */
	
user_mode:
	move.l	4(%a7),%a0	/* move the old ssp location to an addrreg */

	move.l	(%a7)+,%d0	/* fetch return address */
	move.l	%d0,2(%a0)	/* and write to old ssp stack frame */

	move.l	%a7,%usp	/* set the usp to the old value */

	move.l	%a0,%a7		/* restore old ssp */
	rte
