/*
 * encoders.h
 ******************************************************************************
 * Copyright (C) 2002  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. http://www.gnu.org
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 * Header file for routine to process shaft encoder input
 */

#ifndef __ENC_H
#define __ENC_H

/******************************************************************************
 * Define constants pertaining to the LS7266R1 chip
 */

#define ENC_kill_time(X) {volatile Ushort _i, _j; for(_i=X;_i;_i--){_j=_i;}}

#define X_REG		(0)
#define Y_REG		(1)

#define DATA_REG	(0)
#define CONTROL_REG	(1)

////// Commands

#define BOTH_CHANNELS	(0x80)	// OR this when applying cmd to both channels

// Reset and Load Signal Decoders (RLD) commands
#define BP_RESET	(0x01)	// 0000 0001  Reset byte pointer
#define CNTR_RESET	(0x02)	// 0000 0010  Reset counter
#define FLAGS_RESET	(0x04)	// 0000 0100  Reset BT, CT, CPT, S flags
#define EFLAG_RESET	(0x06)	// 0000 0110  Reset E flag
#define XFER_PRE_CNTR	(0x08)	// 0000 1000  Transfer Preset Reg to CNTR
#define XFER_CNTR_OL	(0x10)	// 0001 0000  Transfer CNTR to Output Latch
#define CLOCK_SETUP	(0x18)	// 0001 1000  Transfer Preset to Filt Prescaler

// Counter Mode Registers (CMR) commands
#define QUAD_X1		(0x28)	// 0010 1000  Binary, Norm count, x1
#define QUAD_X2		(0x30)	// 0011 0000  Binary, Norm count, x2
#define QUAD_X4		(0x38)	// 0011 1000  Binary, Norm count, x4

// Input Output Control (IOC) commands
#define INPUT_SETUP	(0x41)	// 0100 0001  Enab Inputs, Flg1=carry

// Index Control Register (IDR) commands
#define INDEX_DISABLE	(0x60)	// 0110 0000  Disable index pulses


// Clock dividers: 104kHz / (9+1) = 10.4 kHz.
// 10.4 kHz / 8 = 1300 Hz (max Quadrature clock rate)
// 1300 / 512 = 2.5 revolutions per second Max Counting Rate

#define CLOCK_DATA	(9)

/******************************************************************************
 */

extern int			ENC_X_coord, ENC_Y_coord ;

extern void ENC_write_byte	(Ushort control_reg, Ushort y_reg, Uchar val) ;
extern void ENC_set_counter	(Ushort y_reg, Ulong val) ;
extern Uchar ENC_read_byte	(Ushort control_reg, Ushort y_reg) ;
extern int ENC_read_counter	(Ushort y_reg) ;
extern int ENC_read_coordinates	(void) ;
extern int ENC_init_hardware	(void) ;

#endif
