/*
 * joystick.c
 ******************************************************************************
 * Copyright (C) 2002  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. http://www.gnu.org
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 * routines to read the joystick port
 */

#include <unistd.h> /* for sleep */
#include <asm/MC68EZ328.h> /* for all I/O port locations */
#include <stdio.h>
#include "types.h"
#include "joystick.h"
#include "aux_driver.h"
#include "asm.h"
#include "process.h"

Uchar JOY_current, JOY_changes ;
char  JOY_AZ, JOY_ALT ;

/******************************************************************************
 */
#define JOY_DELAY 20 // Anything <=~10 means garbage gets read

int
JOY_scan_inputs (void)
{
  Ulong ssp ;
  Uchar in, current_val ;

  /* drive the polling lines to 0. The port should read 0 */
  ssp = supervisor_mode() ;
  AUX_write_aux_bit (AUX_JOY_A, 0, True) ;
  AUX_write_aux_bit (AUX_JOY_B, 0, True) ;
  JOY_kill_time (JOY_DELAY) ;
  in = PCDATA & 0x0f ;

  if (in != 0x00)
  {
    user_mode (ssp) ;
    PR_log_msg (ERR, "Port reads %0x instead of 0\n", in) ;
    _exit (30) ;
  }

  /* Now poll the port one half at a time */
  AUX_write_aux_bit (AUX_JOY_A, 1, True) ;
  JOY_kill_time (JOY_DELAY) ;
  current_val = PCDATA & 0x0f ;
  AUX_write_aux_bit (AUX_JOY_A, 0, True) ;
  current_val <<= 4 ;

  AUX_write_aux_bit (AUX_JOY_B, 1, True) ;
  JOY_kill_time (JOY_DELAY) ;
  in = PCDATA & 0x0f;
  AUX_write_aux_bit (AUX_JOY_B, 0, True) ;
  current_val |= in ;
  user_mode (ssp) ;

  JOY_changes = JOY_current ^ current_val ;
  JOY_current = current_val ;

#ifdef JOY_DEBUG
  if (JOY_changes & JOY_UP_BIT)
    PR_log_msg (NORM, "UP %s\n", (JOY_current & JOY_UP_BIT) ? "ON" : "OFF") ;

  if (JOY_changes & JOY_DOWN_BIT)
    PR_log_msg (NORM, "DOWN %s\n", (JOY_current & JOY_DOWN_BIT) ? "ON" : "OFF") ;

  if (JOY_changes & JOY_LEFT_BIT)
    PR_log_msg (NORM, "LEFT %s\n", (JOY_current & JOY_LEFT_BIT) ? "ON" : "OFF") ;

  if (JOY_changes & JOY_RIGHT_BIT)
    PR_log_msg (NORM, "RIGHT %s\n", (JOY_current & JOY_RIGHT_BIT) ? "ON" : "OFF") ;

  if (JOY_changes & JOY_LIGHT_BIT)
    PR_log_msg (NORM, "LIGHT %s\n", (JOY_current & JOY_LIGHT_BIT) ? "ON" : "OFF") ;

  if (JOY_changes & JOY_MODE_BIT)
    PR_log_msg (NORM, "MODE %s\n", (JOY_current & JOY_MODE_BIT) ? "ON" : "OFF") ;

  if (JOY_changes & JOY_ENTER_BIT)
    PR_log_msg (NORM, "ENTER %s\n", (JOY_current & JOY_ENTER_BIT) ? "ON" : "OFF") ;
#endif

  return JOY_changes ;
}

/******************************************************************************
 */

void
JOY_get_direction (void)
{
  char x, y ;

  x = y = 0 ;
  if (JOY_current & (JOY_UP_BIT | JOY_DOWN_BIT | JOY_LEFT_BIT | JOY_RIGHT_BIT))
  {
    if (JOY_current & JOY_LEFT_BIT) 
      x = -1 ;
    else if (JOY_current & JOY_RIGHT_BIT)
      x = 1 ;

    if (JOY_current & JOY_UP_BIT) 
      y = 1 ;
    else if (JOY_current & JOY_DOWN_BIT)
      y = -1 ;
  }

  #ifdef JOY_DEBUG
    PR_log_msg (NORM, "JOY_AZ:%d, JOY_ALT:%d\n", JOY_AZ, JOY_ALT) ;
  #endif

  if ((x == JOY_AZ) && (y == JOY_ALT))
    return ;

  JOY_AZ = x ;
  JOY_ALT = y ;
}

