/*
 * menu.h
 ******************************************************************************
 * Header file for menu parsing code
 ******************************************************************************
 * Copyright (C) 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#ifndef __MENU_H
#define __MENU_H

///////////////////////////////////////////////////////////////////////////////
// Data structures
//
// Using the following structure, a menu tree may be defined.
//
// up, down, left, right: These menu pointer fields are used to navagate around
//		the tree. Setting the values to NULL indicate there is no more
//		navagation in that direction.
//
//		Setting the "down" field to NULL has a special meaning. Heading
//		downwards when the value is NULL indicates that the value is
//		going to be edited.
//
//		The left and right fields loop around in a circular manner. If
//		there is only one menu item at a level, then left and right
//		should both point to the current entry.
//
// name:	This is the name of the menu item. KEEP THIS FIELD TO A MINIMUM
//		Keep to <= 6 chars and there'll be no trouble. If > 6, then
//		the TWO names to the left and TWO names to the right must be
//		shorter than 12 characters when added.

#define NAME_LEN 8

struct _menu
{
  struct _menu	*up, *down, *left, *right ;	// Menu parsing options
  char	name [NAME_LEN] ;			// name of this menu item
  void	(*disp)(struct _menu *, char *) ;	// check cursor & display value
  void	(*edit)(struct _menu *, char) ;		// change the value
  int	min, max ;				// range of allowed values
  int	value ;					// present value
  void	*other ;				// other information
} ;

typedef struct _menu MENU ;
typedef void DISP_DEF (MENU *, char *) ;
typedef void EDIT_DEF (MENU *, char) ;
typedef int  COM_DEF  (const int val) ;

///////////////////////////////////////////////////////////////////////////////

enum _state
{
  MEN_LOOK,
  MEN_EDIT
} ;

extern MENU	*MEN_current ;
extern char	MEN_cursor ;
extern Uchar	MEN_state ;
extern char	MEN_name [] ;

extern MENU	men1 ; // This is always the main menu item
extern MENU	tzone, location, go_to, set_home ;
extern MENU	messier, ngc, ugc, pgc, sao, hr, hd, ppm, ic, align, planet ;
extern MENU	place, manual ;
extern MENU	moon, mercury, venus, mars, jupiter, saturn, uranus, neptune ;

///////////////////////////////////////////////////////////////////////////////
// Templates

extern void	MEN_process (void) ;
extern COM_DEF	MEN_null_routine ;
extern COM_DEF	MEN_lookup_database ;

extern DISP_DEF MEN_disp_dummy ;
extern EDIT_DEF MEN_edit_dummy ;

extern DISP_DEF MEN_disp_int ;
extern EDIT_DEF MEN_edit_int ;

extern DISP_DEF MEN_disp_time ;
extern EDIT_DEF MEN_edit_time ;

extern DISP_DEF MEN_disp_tz ;
extern EDIT_DEF MEN_edit_tz ;

extern DISP_DEF MEN_disp_home ;
extern EDIT_DEF MEN_edit_home ;

extern DISP_DEF MEN_disp_file ;
extern EDIT_DEF MEN_edit_file ;
extern COM_DEF	MEN_comit_lon_lat ;

extern DISP_DEF MEN_disp_moon ;
extern EDIT_DEF MEN_edit_moon ;

extern DISP_DEF MEN_disp_plan ;
extern EDIT_DEF MEN_edit_plan ;

extern DISP_DEF MEN_disp_align ;
extern EDIT_DEF MEN_edit_align ;

#endif
