/*
 * motors.h
 ******************************************************************************
 * Copyright (C) 2002  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. http://www.gnu.org
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 * Header file for motor driver code
 */

#ifndef __MOTOR_H
#define __MOTOR_H

/******************************************************************************
 */

#define MOT_kill_time(X) {volatile Ushort _i, _j; for(_i=X;_i;_i--){_j=_i;}}

/******************************************************************************
 */

#define MOT_MAX_SPEED		(10)	// Number of entries in the speed table
#define AZ_ARCSEC_PER_REV	(21000)	// arc seconds per turn of the stepper motor
#define ALT_ARCSEC_PER_REV	(21000)	// arc seconds per turn of the stepper motor
#define STEPS_PER_REV		(400)	// half steps per revolution of the stepper motor

enum _MOT_BIT_VALUE
{
  MOT_DIRECTION = 0,
  MOT_HALF_STEP_MODE,
  MOT_C,
  MOT_STEP_A,
  MOT_RUN_MODE,
  MOT_B,
} ;

enum _MOT_DIR
{
  MOT_FORWARD,
  MOT_REVERSE
} ;

struct _MOT_SPEED_STRUCT
{
  Uchar	 bits ;
  Ushort rate ;
} ;

typedef enum _MOT_BIT_VALUE MOT_BIT_VALUE ;
typedef enum _MOT_DIR MOT_DIR ;
typedef struct _MOT_SPEED_STRUCT SPEED_STRUCT ;

/******************************************************************************
 */

extern SPEED_STRUCT MOT_speed [MOT_MAX_SPEED] ;

extern void	MOT_msg (const Ulong steps_per_sec, const MOT_DIR dir, const char alt_az) ;
extern void	MOT_set_run_speed (Ushort speed, Uchar alt_az, MOT_DIR direction) ;
extern void	MOT_advance_one_step (Uchar alt_az, MOT_DIR direction, BOOL half_step) ;
extern void	MOT_process (const Ushort setting) ;
extern int	MOT_motor_process (const char *param) ;

#endif


