/*
 * readline.c
 ******************************************************************************
 * Copyright (C) 2002 Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#include <stdio.h>
#include "types.h"
#include "readline.h"

/******************************************************************************
 * RL_read_line
 *
 * Reads a line of text from an open file. The newline is not included in the
 * string that is returned.
 *
 * If the end of file is encountered, return FALSE, otherwise return TRUE
 *
 * The length of the array of characters must be "maxlen+1" bytes or longer.
 */

int
RL_read_line (char *line, const Ushort maxlen, FILE *f)
{
    int     ch ;
    short   i ;
    char    *p ;

    for (ch=0, p=line ; (ch != EOF) && (p == line) ;)
    {
      p = line ;
      i = maxlen ;

      while (i && ((ch = fgetc(f)) != EOF) && (ch != '\n'))
      {
	*p++ = (char)ch ;
	i-- ;
      }
    }

    *p = '\0' ;
    return (p != line) ;
}
