/*
 * access_data.c
 ******************************************************************************
 * Copyright (C) 2002 Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "types.h"
#include "process.h"
#include "database.h"
#include "sort.h"
#include "access_data.h"

///////////////////////////////////////////////////////////////////////////////
DB_REF DB_ref [DB_NUM_TYPES] ;

const char DB_filename[] = "COMP.dat" ;
///////////////////////////////////////////////////////////////////////////////
FILE *
open_database (void)
{
  FILE *f ;
  char name[50] ;

  strcpy (name, FILE_PREFIX) ;
  strcat (name, DB_filename) ;

  if (((f=fopen(name, "r")) == (FILE *)0) &&
      ((f=fopen(DB_filename, "r")) == (FILE *)0))
  {
    PR_log_msg (ERR, "Can't open %s: %s", DB_filename, strerror(errno)) ;
    exit (50) ;
  }

  #ifdef BASIC_DEBUG
    PR_log_msg (SCREEN, "Opened file: %s", DB_filename) ;
  #endif

  return f ;
}
///////////////////////////////////////////////////////////////////////////////
int
cmp_DB_LIST (void *a, void *b)
{
  if (((DB_LIST *)a)->id > ((DB_LIST *)b)->id)
    return 1;

  if (((DB_LIST *)a)->id < ((DB_LIST *)b)->id)
    return -1;

  return 0 ;
}
///////////////////////////////////////////////////////////////////////////////
void
DB_initialise(void)
{
  FILE *f ;
  OBJECT_INFO o ;
  long pos ;
  Ulong i, j ;
  DB_LIST *p ;
  void **ptr ;
  NUM_ENTRIES num ;

  f = open_database () ;
  DB_read_header (f, &num) ;

  #ifdef BASIC_DEBUG
    PR_log_msg (SCREEN, "Read initial sizes") ;

    PR_log_msg (SCREEN, "Mes=%ld, UGC=%ld, NGC=%ld, HR=%ld, IC=%ld",
		num.messier, num.ugc, num.ngc, num.hr, num.ic) ;

    PR_log_msg (SCREEN, "SAO=%ld, PGC=%ld, HD=%ld, PPM=%ld, ALIGN=%ld",
		num.sao, num.pgc, num.hd, num.ppm, num.align) ;
  #endif

  // Now, allocate memory to hold pointers into the database file.
  if (((DB_ref[DB_MES].list = calloc(num.messier, sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_UGC].list = calloc(num.ugc, sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_NGC].list = calloc(num.ngc, sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_HR].list  = calloc(num.hr,  sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_IC].list  = calloc(num.ic,  sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_SAO].list = calloc(num.sao, sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_PGC].list = calloc(num.pgc, sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_HD].list  = calloc(num.hd,  sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_PPM].list = calloc(num.ppm, sizeof(DB_LIST))) == NULL) ||
      ((DB_ref[DB_ALIGN].list = calloc(num.align, sizeof(DB_LIST))) == NULL) ||

      ((DB_ref[DB_MES].p = calloc(num.messier, sizeof(void *))) == NULL) ||
      ((DB_ref[DB_UGC].p = calloc(num.ugc, sizeof(void *))) == NULL) ||
      ((DB_ref[DB_NGC].p = calloc(num.ngc, sizeof(void *))) == NULL) ||
      ((DB_ref[DB_HR].p  = calloc(num.hr,  sizeof(void *))) == NULL) ||
      ((DB_ref[DB_IC].p  = calloc(num.ic,  sizeof(void *))) == NULL) ||
      ((DB_ref[DB_SAO].p = calloc(num.sao, sizeof(void *))) == NULL) ||
      ((DB_ref[DB_PGC].p = calloc(num.pgc, sizeof(void *))) == NULL) ||
      ((DB_ref[DB_HD].p  = calloc(num.hd,  sizeof(void *))) == NULL) ||
      ((DB_ref[DB_PPM].p = calloc(num.ppm, sizeof(void *))) == NULL) ||
      ((DB_ref[DB_ALIGN].p = calloc(num.align, sizeof(void *))) == NULL))
  {
    fclose (f) ;
    PR_log_msg (ERR, "Can't allocate mem") ;
    exit (51) ;
  }

  // Now pass through the file and load the memory

  for (i=0 ; i < DB_NUM_TYPES ; i++)
    DB_ref[i].num = 0 ;

  for (;;)
  {
    pos = ftell (f) ; // get the file position indicator

    if (DB_read_object_info(&o, f, 0) != 1)
      break ;

    if (o.bit_fields & BIT_MESSIER)
    {
      p = &DB_ref[DB_MES].list[DB_ref[DB_MES].num++] ;
      p->id = o.messier ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_NGC)
    {
      p = &DB_ref[DB_NGC].list[DB_ref[DB_NGC].num++] ;
      p->id = o.ngc ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_UGC)
    {
      p = &DB_ref[DB_UGC].list[DB_ref[DB_UGC].num++] ;
      p->id = o.ugc ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_HR)
    {
      p = &DB_ref[DB_HR].list[DB_ref[DB_HR].num++] ;
      p->id = o.hr ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_IC)
    {
      p = &DB_ref[DB_IC].list[DB_ref[DB_IC].num++] ;
      p->id = o.ic ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_SAO)
    {
      p = &DB_ref[DB_SAO].list[DB_ref[DB_SAO].num++] ;
      p->id = o.sao ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_PGC)
    {
      p = &DB_ref[DB_PGC].list[DB_ref[DB_PGC].num++] ;
      p->id = o.pgc ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_HD)
    {
      p = &DB_ref[DB_HD].list[DB_ref[DB_HD].num++] ;
      p->id = o.hd ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_PPM)
    {
      p = &DB_ref[DB_PPM].list[DB_ref[DB_PPM].num++] ;
      p->id = o.ppm ;
      p->file_offset = pos ;
    }

    if (o.bit_fields & BIT_ALIGNMENT)
    {
      p = &DB_ref[DB_ALIGN].list[DB_ref[DB_ALIGN].num] ;
      p->id = DB_ref[DB_ALIGN].num++ ;
      p->file_offset = pos ;
    }
  }

  fclose (f) ;

  // create the lists of sorted pointers
  for (i=0 ; i < DB_NUM_TYPES ; i++)
  {
    ptr = DB_ref[i].p ;

    for (j=0 ; j < DB_ref[i].num ; j++)
      ptr[j] = &DB_ref[i].list[j] ;

    SOR_qsort (ptr, cmp_DB_LIST, 0, DB_ref[i].num-1) ;
  }
}

///////////////////////////////////////////////////////////////////////////////
int
DB_lookup (const DB_TYPE type, const int num, OBJECT_INFO *o, const int brief)
{
  FILE *f ;
  Ulong ref ;
  DB_LIST key ;
  void **ptr ;

  #ifdef BASIC_DEBUG
    if (type >= DB_NUM_TYPES)
    {
      PR_log_msg (ERR, "DB_lookup: illegal type %d", type) ;
      return 1 ;
    }
  #endif

  key.id = num ;

  if ((ref=SOR_search (DB_ref[type].p, cmp_DB_LIST, DB_ref[type].num, (void *)&key)) == SOR_FAIL)
  {
    PR_log_msg (ERR, "Can't find type=%d, id=%d", (int)type, num) ;
    return 1 ;
  }

  f = open_database () ;
  ptr = DB_ref[type].p ;

  if (fseek (f, ((DB_LIST *)(ptr[ref]))->file_offset, SEEK_SET))
  {
    fclose (f) ;
    PR_log_msg (ERR, "DB_lookup: File seek error: %s", strerror(errno)) ;
    return 1 ;
  }

  if (DB_read_object_info(o, f, brief) != 1)
  {
    fclose (f) ;
    PR_log_msg (ERR, "DB_lookup read error: type=%d, id=%d", type, num) ;
    return 1 ;
  }

  fclose (f) ;
  return 0 ;
}
