/*
 * process_setup.h - To set up process.c / process.h
 ******************************************************************************
 * Copyright (C) 2001  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#ifndef __SETUP_H
#define __SETUP_H

/*
 * define the following according to the capabilities of your OS
 *
 * HAVE_FORK: if not defined, implies VFORK is available instead
 * HAVE_SHM: if not defined, malloc used instead
 * HAVE_TIMEKEEPER: to include the timekeeper process and related routines
 * HAVE_LOG_PROCESS: if not defined, log messages sent directly to fprintf
 */

// #define BASIC_DEBUG
// #define MESSAGE_TRACE
// #define SIG_HANDLER_DEBUG
// #define TIMER_DEBUG
// #define PARANOID

// #define HAVE_FORK
// #define HAVE_SHM
#define HAVE_TIMEKEEPER
// #define HAVE_LOG_PROCESS

#define LOG_MSG_SIZE	100	/* max characters in a log message */

#define	TICK_FREQ	10	/* Hz (granularity of timer service) */
#define NUM_TIMERS	40	/* max number of simultaneous timers */
#define DELAY_PERIOD	50000 	/* microseconds-long enough to cause a swap */

#define PROCESS_NAME_SIZE (11)	/* max length of process name */

#define USER_DEFINED_GLOBALS	/* declare application specific globals here */

#define MAX_NUM_PROC	(20)	/* The size of the process table */

enum MY_MESSAGE_TYPES
{
  STEPPER_SPEED = NUM_PROC_MSGS,
  LCD_MESSAGE,
  LCD_CURSOR,
  INITIALISE,

  NUM_MY_MESSAGES
} ;

typedef enum	MY_MESSAGE_TYPES	MSG_TYPE ;

/******************************************************************************
 * declare application specific globals here
 */

#define USER_DEFINED_GLOBALS \
  int lcd_msgq ;\
  int alt_msgq ;\
  int az_msgq ;\
  int time_offset ;

#endif
