/*
 * test_menu.c
 ******************************************************************************
 * Copyright (C) 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sched.h>
#include <time.h>
#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <math.h>

#include "types.h"
#include "menu.h"
#include "joystick.h"
#include "astro.h"
#include "process.h"
#include "access_data.h"
#include "telescope.h"

///////////////////////////////////////////////////////////////////////////////

#define DO_MOVEMENT(X) {JOY_changes = JOY_current = (X) ; MEN_process() ; }

Uchar	JOY_current, JOY_changes ;
SHARED_GLOBAL_STORAGE	*Global ;

///////////////////////////////////////////////////////////////////////////////
void
TEL_lcd_msg (const Uchar line, const Uchar delay, const char *fmt, ...)
{
  va_list	args ;
  char str[100] ;

  va_start(args, fmt) ;
  vsprintf (str, fmt, args) ;
  va_end(args) ;

  printf("Line %d: <%s> len:%d\n", line, str, strlen(str)) ;
}

void
TEL_lcd_cur (const Uchar line, const Uchar cursor)
{
  int i ;

  printf("Cursor:  ") ;

  if (cursor > 20)
  {
    printf ("Illegal cursor location: %d\n", (int)cursor) ;
    return ;
  }

  for (i=cursor ; i ; i--)
    printf (" ") ;

  printf ("^  (line=%d)\n", line) ;
}

void
PR_sleep (const Ushort interval)
{
}

void
TEL_one_sec_msg (const long mes_type)
{
}
///////////////////////////////////////////////////////////////////////////////

int
main()
{
  DB_initialise() ; // populate the databases

  MEN_current = &place ; // set LON / LAT to the default location
  TEL_lcd_msg (0, 0, "Setting default loc") ;
  MEN_disp_file (MEN_current, NULL) ;
  MEN_edit_file (MEN_current, 0) ;

  MEN_current = &men1 ; // initialise menu to the top
  MEN_cursor = 0 ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // time zone
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ; // time zone
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // location
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // goto
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // Messier
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ; // select M1
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ; // goto
  printf ("\n") ;


  DO_MOVEMENT (JOY_DOWN_BIT) ; // Messier
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;
  DO_MOVEMENT (JOY_UP_BIT) ;
  printf ("\n") ;
  DO_MOVEMENT (JOY_UP_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ; // select M21
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ; // goto
  printf ("\n") ;


  DO_MOVEMENT (JOY_LEFT_BIT) ; // moon
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // planet
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // mercury
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // venus
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // mars
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // jupiter
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // RA/DEC
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // ALT/AZ
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // RA/DEC
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // saturn
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // uranus
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // neptune
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_UP_BIT) ; // planet
  printf ("\n") ;

  DO_MOVEMENT (JOY_UP_BIT) ; // goto
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // location
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // place
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // open the file "place.info"
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // line 2
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // line 3
  printf ("\n") ;

  DO_MOVEMENT (JOY_ENTER_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_UP_BIT) ; // location
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // goto
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // Messier
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // Moon
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // Planet
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // Align
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // star #1
  printf ("\n") ;

  DO_MOVEMENT (JOY_RIGHT_BIT) ; // RA/DEC
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // star #2
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // star #3
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // star #4
  printf ("\n") ;

  DO_MOVEMENT (JOY_DOWN_BIT) ; // star #5
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // RA/DEC
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ; // ALT/AZ
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;

  DO_MOVEMENT (JOY_LEFT_BIT) ;
  printf ("\n") ;


  return 0 ;
}
