/*
 * parse.h
 ******************************************************************************
 * Copyright (C) 2002  Stefan Keller-Tuberg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2 as published
 * by the Free Software Foundation
 *
 * These conditions include but are not limited to the requirement that the
 * origianal author is acknowledged in any reused, modified or adapted
 * versions of any part of this code and that any system containing reused or
 * modified parts of this code are issued under the GNU General Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. http://www.gnu.org
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************
 */

#ifndef __PARSE_H
#define __PARSE_H

enum _OPT_TYPE
{
  BOOLEAN,
  INT,
  STRING,
  FILENAME
} ;

typedef enum _OPT_TYPE OPT_TYPE ;

struct _OPTION
{
  char     *op_str ;
  char     *description ;
  OPT_TYPE type ;
  void     *var ;
  int      default_int ; /* if string, 0:default to null, !=0:default to string */
  char     *default_string ;
  int      min ; /* min int */
  int      max ; /* max int or max length of string */
  int      hidden ; /* 0 if visible, 1 if hidden */
} ;

typedef struct _OPTION OPTION ;

/******************************************************************************
 * Global variables
 */

extern int	Copyleft ;
extern int	Verbose ;
extern int	Error ;
extern int	Help ;

#define DECLARE_PARSE_OPTIONS \
  {"c", "Display copyleft information", BOOLEAN, &Copyleft}, \
  {"v", "Be verbose", BOOLEAN, &Verbose}, \
  {"?", "Display help message", BOOLEAN, &Help}, \
  {""} /* no need to terminate this with a comma */

#define PROCESS_PARSE_OPTIONS(_PROG_NAME) \
{\
  if (Copyleft != 0)\
    PA_copyleft (_PROG_NAME) ;\
\
  if (Verbose)\
    PA_output_config() ;\
}

#define PROCESS_FINAL_OPTIONS(_ERROR, _PROG_NAME, _DESCRIPTION) \
{\
  if ((_ERROR) || Help)\
  {\
    STR_extract_filename (_PROG_NAME) ;\
    PA_usage (Filename, _DESCRIPTION, "") ;\
    exit (0) ;\
  }\
}

/******************************************************************************
 */

extern OPTION opt[] ;
extern int  PA_parse (int *argc, char ***argv) ;
extern void PA_usage (char *progname, char *use, char *extra) ;
extern void PA_output_config (void) ;
extern void PA_dump_options (char **argv) ;
extern void PA_copyleft (char *name) ;

#endif
